#include <iostream>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>

#include "lw14.h"

using namespace std;

//Get broadcast DALI address
int lw14_SetBroadcastAddress(unsigned char mode)
{
	return (0xFE | mode);
}

//Get group address
int lw14_SetGroupAddress(int group, unsigned char mode)
{
	return (0x80 | ((group & 16) << 1) | mode);
}

//Get short address
int lw14_SetShortAddress(int device, unsigned char mode)
{
	return (((device & 63) << 1) | mode);
}

int lw14_GetStatus(int device)
{
    if(ioctl(device, I2C_SLAVE, LW14_I2CADR) >= 0)
        return i2c_smbus_read_byte_data(device, LW14_REG_STATUS);
    else
    {
        printf("Error when read. ioctl() I2C not found\n");
        return 0;
    }
}

int lw14_GetResult(int device)
{
    if(ioctl(device, I2C_SLAVE, LW14_I2CADR) >= 0)
    {
        i2c_smbus_write_byte(device, 0x01); //Set command register as read register
        usleep(100*1000);                   //wait for i2c a moment
        return i2c_smbus_read_byte_data(device, LW14_REG_CMD); //read data from command register
    }
    else
    {
        printf("Error when read. ioctl() I2C not found\n");
        return 0;
    }
}

int lw14_WaitForValidReply(int device)
{
    unsigned char res = 0x00;

    while(true)
    {
        res = 0;
        usleep(1000); //wait to reduce CPU 

        res = lw14_GetStatus(device);

        if((res & LW14_STATE_BUSY) == 0) //wait until busy flag is gone
        {
            //wait until busy flag is ZERO
            if((res & (LW14_STATE_VALID | LW14_STATE_1BYTE)) == (LW14_STATE_VALID | LW14_STATE_1BYTE)) //valid result
                break;
            else if(res == 0) //nobody answered -> ESC
                break;
        }
    }

    return res;
}


int lw14_Send(int device, int adr, int value)
{
    int res = 0;
    unsigned char data[] = {(unsigned char) adr, (unsigned char) value};

    if(ioctl(device, I2C_SLAVE, LW14_I2CADR) >= 0)
    {
        //device handle, register, datacount, data store
        res = i2c_smbus_write_i2c_block_data(device, 0x01, 2, &data[0]) ;

        if(res < 0)
            printf("Something went wrong when write. Maybe wrong I2C-Address?\n");
    }
    else
        printf("Error when write. ioctl() I2C not found\n");

    return res;
}


//Query only work with short address on DALI
int lw14_GetQueryValue(int device, int adr, int value)
{
    //Send DALI query command
    lw14_Send(device, adr, value);

    //Wait for valid reply
    lw14_WaitForValidReply(device);

    //Read data from command register
    return lw14_GetResult(device);
}


int main(int argc, char** argv)
{
    int device = 0;
    unsigned long funcs;
    int dali_adr = 0;
    int result = 0;


    //Open I2c interface
    if((device = open("/dev/i2c-1", O_RDWR)) < 0)
    {
        perror("Failed to open I2C bus\n");
        exit(1);
    }

    if(ioctl(device, I2C_FUNCS, &funcs) < 0)
    {
        perror("ioctl() I2C_FUNCS failed\n");
        exit(1);
    }

    //get a query value
    //Send ./lw14_query <dali (INT)> <quer(INT)>
    dali_adr = lw14_SetShortAddress(atoi(argv[1]), LW14_MODE_CMD);
    result = lw14_GetQueryValue(device, dali_adr, atoi(argv[2]));

    printf("Query result: %d (0x%02x)\n", result, result);

    //Close interface!
    close(device);

    return 0;
}
