#include <iostream>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>

#include "lw14.h"

using namespace std;

int Send(int, int, int);
int SetI2CAddress(int, int);
unsigned char GetBroadcastAddress(unsigned char);
unsigned char GetGroupAddress(int, unsigned char);
unsigned char GetShortAddress(int, unsigned char);


unsigned char GetBroadcastAddress(unsigned char mode)
{
	return (0xFE | mode);
}

unsigned char GetGroupAddress(int group, unsigned char mode)
{
	return (0x80 | ((group & (DALI_MAX_GROUPS-1)) << 1) | mode);
}

unsigned char GetShortAddress(int device, unsigned char mode)
{
	return (((device & (DALI_MAX_ADDRESS-1)) << 1) | mode);
}

int Send(int device, int adr, int value)
{
    int res = 0;
    unsigned char data[] = {(unsigned char) adr, (unsigned char) value};

    if(ioctl(device, I2C_SLAVE, LW14_I2C_ADR) >= 0)
    {
        //i2c_smbus_write_i2c_block_data(device handle, register, datacount, data array)
        res = i2c_smbus_write_i2c_block_data(device, LW14_REG_CMD , 2, &data[0]);

        if(res < 0)
            printf("Something went wrong when write.\n");
    }
    else
        printf("Error when write. ioctl() I2C not found.\n");

    return res;
}

int SetI2CAddress(int device, int adr)
{
    int res = 0;
    //negated addr value to check by the chip
    unsigned char data[] = {(unsigned char) adr, (unsigned char) ~(adr)};

    if(ioctl(device, I2C_SLAVE, LW14_I2C_ADR) >= 0)
    {
        res = i2c_smbus_write_i2c_block_data(device, LW14_REG_ADDR, 2, &data[0]);

        if(res < 0)
            printf("Something went wrong when write.\n");
    }
    else
        printf("Error when write. ioctl() I2C not found.\n");

    return res;
}

int main(int argc, char** argv)
{
    int result = 0;
    int device;
    unsigned long funcs;
    unsigned char dali_adr;

    //Open I2C (1) interface
    if((device = open("/dev/i2c-1", O_RDWR)) < 0)
    {
        perror("Failed to open I2C bus\n");
        exit(1);
    }

    if(ioctl(device, I2C_FUNCS, &funcs) < 0)
    {
        perror("ioctl() I2C_FUNCS failed\n");
        exit(1);
    }

    //Send DALI DACP value to broadcast 
    unsigned char dacp = 155;
    dali_adr = GetBroadcastAddress(DALI_MODE_DACP);
    result = Send(device, dali_adr, dacp);

    close(device);

    return result;
}
