// compile: gcc <code>.c -o <appname> -l iowkit

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>

#include "iowkit.h"
#include "lw14.h"

#define FALSE 0
#define TRUE 1

int SetDeviceMode(IOWKIT_HANDLE, unsigned char);
int Send(IOWKIT_HANDLE, unsigned char, unsigned char);
int GetBroadcastAddress(unsigned char);
int GetGroupAddress(int, unsigned char);
int GetShortAddress(int, unsigned char);


int SetDeviceMode(IOWKIT_HANDLE handle, unsigned char mode)
{
	IOWKIT_SPECIAL_REPORT report;

	if(handle != NULL)
	{
		report.ReportID = 0x01;  //I2C Mode - Report ID
		report.Bytes[0] = mode;  //Enable 0x01 or Disable 0x00
		report.Bytes[1] = 0x00;  //Flags
		report.Bytes[2] = 0x00;  //Timeout

		IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE);
		//No IowKitRead unnecessary
		return 1;
	}

	return -1;
}

int Send(IOWKIT_HANDLE handle, unsigned char v1, unsigned char v2)
{
	IOWKIT_SPECIAL_REPORT report;

	if(handle != NULL)
	{
		//Send DALI Command
		memset(&report, 0x00, IOWKIT_SPECIAL_REPORT_SIZE);

		report.ReportID = 0x02;	//I2C Write
		report.Bytes[0] = 0xC4;	//Start + Stopbit, 4 byte data
		report.Bytes[1] = 0x40;	//I2C device address (8 Bit)
		report.Bytes[2] = LW14_REG_CMD;	//Data register
		report.Bytes[3] = v1;	//Data value 1
		report.Bytes[4] = v2;	//Data value 2

		if(IowKitWrite(handle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE) == IOWKIT_SPECIAL_REPORT_SIZE)
		{
			if(IowKitRead(handle, IOW_PIPE_SPECIAL_MODE, (char*) &report, IOWKIT_SPECIAL_REPORT_SIZE) == IOWKIT_SPECIAL_REPORT_SIZE)
				return 1;
			else
				return 0;
		}
		else
			return 0;
	}

	return -1;
}

int GetBroadcastAddress(unsigned char mode)
{
	return (0xFE | mode);
}

int GetGroupAddress(int group, unsigned char mode)
{
	return (0x80 | ((group & (DALI_MAX_GROUPS-1)) << 1) | mode);
}

int GetShortAddress(int device, unsigned char mode)
{
	return (((device & (DALI_MAX_ADDRESS-1)) << 1) | mode);
}


int main(int argc, char* argv[]) 
{
	unsigned short sn[9];
	char snt[9];

	IOWKIT_HANDLE ioHandle = IowKitOpenDevice();	// Open/connect IOWKIT

	if(ioHandle != NULL)
	{
		//Output Serial Number
		IowKitGetSerialNumber(ioHandle, sn);

		for (int j = 0; j < 9; j++)
			snt[j] = sn[j];

		printf("S/N: %s\n", snt); 

		//Enable I2C Mode 
		SetDeviceMode(ioHandle, 0x01);

		//Send DALI Command
		unsigned char dali = GetBroadcastAddress(DALI_MODE_DACP);
		unsigned char dacp = 50;
		Send(ioHandle, dali, dacp);

		//Disable I2C Mode
		SetDeviceMode(ioHandle, 0x01);
	}
	else
		printf("Kein IO-Warrior gefunden");

	//Don't forget to close API and devices at the end
	IowKitCloseDevice(ioHandle);

	return 0;
}


